
require 'sketchup.rb'
require 'plugindialog.rb'
require 'wxSketchup'
include Wx


ID_ORIENTATION = 1001
ID_INTERVAL = 1002
ID_WIDTH = 1003

#
# simple app for running dialog and creating grid
#

class PlaneGridDialog < PluginDialog
    def initialize(parent)
        super(parent, "Create Grid on Plane")

		# grid plane radio buttons
        choices = [
            "X-Y Plane",
            "Y-Z Plane",
            "Z-X Plane",
        ]
        @orientation = RadioBox.new( self, ID_ORIENTATION, "Grid Orientation", Point.new(-1, -1), Size.new(-1, -1), choices, 1, RA_SPECIFY_COLS )

        # width label and text field
        width_label = StaticText.new(self, -1, "Width:" )
        @width_box = TextCtrl.new( self, ID_WIDTH )
		sizerWidth = BoxSizer.new(HORIZONTAL)
        sizerWidth.add(width_label, 0, ALIGN_RIGHT | ALIGN_CENTER_VERTICAL | ALL)
        sizerWidth.add(@width_box, 0, ALIGN_LEFT | ALIGN_CENTER_VERTICAL | ALL)

		# grid spacing label and text field
        spacing_label = StaticText.new(self, -1, "Grid Spacing:" )
        @interval_box = TextCtrl.new( self, ID_INTERVAL )
		sizerSpacing = BoxSizer.new(HORIZONTAL)
        sizerSpacing.add(spacing_label, 0, ALIGN_RIGHT | ALIGN_CENTER_VERTICAL | ALL)
        sizerSpacing.add(@interval_box, 0, ALIGN_LEFT | ALIGN_CENTER_VERTICAL | ALL)

		# combine two text fields in a sizer
        sizerRight = BoxSizer.new(VERTICAL)
        sizerRight.add(sizerWidth, 0, ALIGN_RIGHT | ALL, 5)
        sizerRight.add(sizerSpacing, 0, ALIGN_RIGHT | ALL, 5)

        # combine radio buttons and text fields into a sizer
        sizerTop = BoxSizer.new(HORIZONTAL)
		sizerTop.add(@orientation, 0, ALIGN_LEFT | ALL, 5)
		sizerTop.add(sizerRight, 0, ALIGN_RIGHT | ALIGN_CENTER_VERTICAL | ALL, 5)

		# Help, Cancel, OK buttons
        @m_btnHelp = Button.new(self, ID_HELP, "Help")
        btnCancel = Button.new(self, ID_CANCEL, "&Cancel")
        btnOk = Button.new(self, ID_OK, "OK")
        sizerBottom = BoxSizer.new(HORIZONTAL)
        sizerBottom.add(btnOk, 0, ALIGN_CENTER | ALL, 5)
        sizerBottom.add(btnCancel, 0, ALIGN_CENTER | ALL, 5)
        sizerBottom.add(@m_btnHelp, 0, ALIGN_CENTER | ALL, 5)

		# combine top and bottom sections with final sizer
        sizerAll = BoxSizer.new(VERTICAL)
		sizerAll.add(sizerTop, 0, ALIGN_CENTER | ALL, 5)
		sizerAll.add(sizerBottom, 0, ALIGN_RIGHT | ALL, 5)
        set_auto_layout(TRUE)
        set_sizer(sizerAll)

        sizerAll.set_size_hints(self)
        sizerAll.fit(self)

        btnOk.set_default()

        #evt_button(-1) {|event| onButton(event) }
    end
end


class FaceGridDialog < PluginDialog
    def initialize(parent)
        super(parent, "Create Grid on Selected Face")

		# grid spacing label and text field
        spacing_label = StaticText.new(self, -1, "Grid Spacing:" )
        @interval_box = TextCtrl.new( self, ID_INTERVAL )
		sizerTop = BoxSizer.new(HORIZONTAL)
        sizerTop.add(spacing_label, 0, ALIGN_RIGHT | ALIGN_CENTER_VERTICAL | ALL)
        sizerTop.add(@interval_box, 0, ALIGN_LEFT | ALIGN_CENTER_VERTICAL | ALL)

		# Help, Cancel, OK buttons
        @m_btnHelp = Button.new(self, ID_HELP, "Help")
        btnCancel = Button.new(self, ID_CANCEL, "&Cancel")
        btnOk = Button.new(self, ID_OK, "OK")
        sizerBottom = BoxSizer.new(HORIZONTAL)
        sizerBottom.add(btnOk, 0, ALIGN_CENTER | ALL, 5)
        sizerBottom.add(btnCancel, 0, ALIGN_CENTER | ALL, 5)
        sizerBottom.add(@m_btnHelp, 0, ALIGN_CENTER | ALL, 5)

		# combine top and bottom sections with final sizer
        sizerAll = BoxSizer.new(VERTICAL)
		sizerAll.add(sizerTop, 0, ALIGN_CENTER | ALL, 5)
		sizerAll.add(sizerBottom, 0, ALIGN_RIGHT | ALL, 5)
        set_auto_layout(TRUE)
        set_sizer(sizerAll)

        sizerAll.set_size_hints(self)
        sizerAll.fit(self)

        btnOk.set_default()

        #evt_button(-1) {|event| onButton(event) }
    end
end


class PluginGridApp < PluginApp

	@@width = 500.feet
	@@interval = 10.feet
	@@plane_selection = 0

	def PluginGridApp.create_grid
		grid_app = PluginGridApp.new
		grid_app.init_frame
		model = Sketchup.active_model
		ss = model.selection
		if ((ss.length == 1) && (ss[0].class == Sketchup::Face))
			# one face selected, create grid on face
			grid_app.create_face_grid
		else
			grid_app.create_plane_grid
		end
	end

	# draw grid aligned with selected face
	def create_face_grid
		# get transform that describes selected face
		model = Sketchup.active_model
		ss = model.selection
		norm = ss[0].normal
		origin = ss[0].edges[0].line[0]
		xaxis = ss[0].edges[0].line[1]	# Vector3D of first edge becomes x axis
		yaxis = xaxis * norm

		xform = Geom::Transformation.new (xaxis, yaxis, norm, origin)
		inv_xform = xform.inverse

		vertices = ss[0].vertices
		normalized_vertices = vertices.collect { |x| inv_xform * x.position }

		x_coords = normalized_vertices.collect { |v| v.x }
		y_coords = normalized_vertices.collect { |v| v.y }
		x_coords.sort!
		y_coords.sort!
		sorted_vertices = vertices.sort {|x,y| x.position.distance(origin) <=> y.position.distance(origin) }
		last = normalized_vertices.length - 1

		face_grid_dialog = FaceGridDialog.new(@@frame)

		face_grid_dialog.set_text_value(ID_INTERVAL, @@interval.to_s)
		##face_grid_dialog.set_help_dialog(xrc_file, 'ID_HELP_FACE_DIALOG')

    	rc = face_grid_dialog.show_modal()
		@@interval = face_grid_dialog.get_text_value(ID_INTERVAL).to_l
		
		face_grid_dialog = nil
		
		if (rc != Wx::ID_OK)
			return
		end

		draw_grid (x_coords[0], x_coords[last],
				y_coords[0], y_coords[last],
				@@interval.to_f, xform)

	end


	# draw grid aligned with one of the drawing planes
	def create_plane_grid

		plane_grid_dialog = PlaneGridDialog.new(@@frame)

		plane_grid_dialog.set_text_value(ID_WIDTH, @@width.to_s)
		plane_grid_dialog.set_text_value(ID_INTERVAL, @@interval.to_s)
		plane_grid_dialog.set_radiobox_index(ID_ORIENTATION, @@plane_selection)
		##plane_grid_dialog.set_help_dialog(xrc_file, 'ID_HELP_AXIS_DIALOG')

    	rc = plane_grid_dialog.show_modal()

		@@plane_selection = plane_grid_dialog.get_radiobox_index(ID_ORIENTATION)
		@@interval = plane_grid_dialog.get_text_value(ID_INTERVAL).to_l
		@@width = plane_grid_dialog.get_text_value(ID_WIDTH).to_l
		@@length = @@width

		plane_grid_dialog = nil

		if (rc != Wx::ID_OK)
			return
		end

		if (@@plane_selection == 0)
			# X-Y plane
			xform = Geom::Transformation.new(Geom::Point3d.new(0,0,0), Geom::Vector3d.new(1,0,0),  Geom::Vector3d.new(0,1,0))
		elsif (@@plane_selection == 1)
			# Y-Z plane
			xform = Geom::Transformation.new(Geom::Point3d.new(0,0,0), Geom::Vector3d.new(0,1,0),  Geom::Vector3d.new(0,0,1))
		else
			# Z-X plane
			xform = Geom::Transformation.new(Geom::Point3d.new(0,0,0), Geom::Vector3d.new(0,0,1),  Geom::Vector3d.new(1,0,0))
		end

		draw_grid (-@@width.to_f/2, @@width.to_f/2,
				-@@length.to_f/2, @@length.to_f/2,
				@@interval.to_f, xform)
	end


	# draw 2D construction grid
	def draw_grid (start_x, stop_x, start_y, stop_y, interval, xform)

		# Start entity creation.
		model = Sketchup.active_model
		model.start_operation "Create Grid"

		entities = model.active_entities
		group = entities.add_group
		entities = group.entities

		step_x = start_x
		while step_x <= stop_x
			start_pt = [step_x, start_y, 0]
			stop_pt = [step_x, stop_y, 0]
			line1 = entities.add_cline(xform*start_pt, xform*stop_pt, "." )
			step_x = step_x + interval
		 end
		start_pt = [stop_x, start_y, 0]
		stop_pt = [stop_x, stop_y, 0]
		line1 = entities.add_cline(xform*start_pt, xform*stop_pt, "." )

		step_y = start_y
		while step_y <= stop_y
			start_pt = [start_x, step_y, 0]
			stop_pt = [stop_x, step_y, 0]
			line1 = entities.add_cline(xform*start_pt, xform*stop_pt, "." )
			step_y = step_y + interval
		 end
		start_pt = [start_x, stop_y, 0]
		stop_pt = [stop_x, stop_y, 0]
		line1 = entities.add_cline(xform*start_pt, xform*stop_pt, "." )

		# Close/commit group
		model.commit_operation
		model.active_view.invalidate
	end
end

# -----------------------------------------------------------------------------

# Add item "Grid" to the Draw menu.
# First check for existing menu load
if( not file_loaded?("cgrid.rb") )

    # Add a separator to the menu, but only once
    add_separator_to_menu("Draw")
    UI.menu("Draw").add_item("Construction Grid") { PluginGridApp.create_grid }
printf( "add menu choice\n")
end

# -----------------------------------------------------------------------------
file_loaded("cgrid.rb")
